(*
	LablPCRE - PCRE[posix] binding for Objective Caml

	author: Robert Roessler
	email: robertr@rftp.com
	web: http://www.rftp.com

--------------------------------------------------------------------------------
Copyright (c) 2005 Robert Roessler
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Robert Roessler nor the names of its contributors
      may be used to endorse or promote products derived from this
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
--------------------------------------------------------------------------------
*)

(* pcre.mli - interface of PCRE[posix] binding for OCaml *)

(** standard PCRE[posix] compiled regexp *)
type regexp

(** standard PCRE[posix] error values *)
type regerror = REG_NOERROR (* non-POSIX and non-PCRE... but useful *)
	| REG_ASSERT | REG_BADBR | REG_BADPAT | REG_BADRPT | REG_EBRACE
	| REG_EBRACK | REG_ECOLLATE | REG_ECTYPE | REG_EESCAPE | REG_EMPTY
	| REG_EPAREN | REG_ERANGE | REG_ESIZE | REG_ESPACE | REG_ESUBREG
	| REG_INVARG | REG_NOMATCH

(** type to hold PCRE[posix] match and/or error state
	N.B. - if a match is found AND captured substrings were requested,
	the searched string is kept in addition to the match results array *)
type t

(** standard PCRE[posix] regexp compile flags *)
type c_flag = REG_ICASE | REG_NEWLINE | REG_DOTALL | REG_EXTENDED | REG_NOSUB

(** standard PCRE[posix] regexp execute flags *)
type x_flag = REG_NOTBOL | REG_NOTEOL

(** compile a PCRE[posix] regexp
	'Failure' raised on invalid regexp *)
val regcomp : ?flags:c_flag list -> string -> regexp

(** execute a PCRE[posix] compiled regexp (FULL)
	use this if you want to capture substrings and/or error state *)
val regexec :
  reg:regexp -> ?n_match:int -> ?pos:int -> ?flags:x_flag list -> string -> t

(** execute a PCRE[posix] compiled regexp (MATCH ONLY)
	use this if you want a quick true/false match query *)
val regmatch : reg:regexp -> ?pos:int -> ?flags:x_flag list -> string -> bool

(** retrieve PCRE[posix] error string
	use with error value from error func and optional compiled regexp *)
val regerror : ?reg:regexp -> regerror -> string

(** return match status *)
val success : t -> bool

(** return error code from regexec *)
val error : t -> regerror

(** return error from regexec as regerror-generated string
	use regerror func directly for the "optional compiled regexp" form *)
val error_string : t -> string

(** return max number of captured substrings *)
val length : t -> int

(** return captured substring i
	0 is entire portion of searched string that matched
	1..n correspond to n [possibly] captured substrings *)
val sub : t -> int -> string

(** return whether substring i was captured *)
val exists : t -> int -> bool

(** return starting offset of captured substring i *)
val sub_start : t -> int -> int

(** return ending offset of captured substring i *)
val sub_end : t -> int -> int

(** object to hold PCRE[posix] match and/or error type ("old" interface)
	create with new regresult <val of type Pcre.t> *)
class regresult :
  t ->
  object
    method success : bool
    method error : regerror
    method length : int
    method substring : int -> string
    method substring_captured : int -> bool
    method substring_start : int -> int
    method substring_end : int -> int
  end
